@@ -23,8 +23,8 @@
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
 define(['jquery', 'core/templates', 'core/notification', 'core/key_codes',
-        'core/custom_interaction_events', 'core/modal_backdrop', 'core/event', 'core/modal_events'],
-     function($, Templates, Notification, KeyCodes, CustomEvents, ModalBackdrop, Event, ModalEvents) {
+        'core/custom_interaction_events', 'core/modal_backdrop', 'core/event', 'core/modal_events', 'core/pending'],
+     function($, Templates, Notification, KeyCodes, CustomEvents, ModalBackdrop, Event, ModalEvents, Pending) {
 
     var SELECTORS = {
         CONTAINER: '[data-region="modal-container"]',
@@ -545,6 +545,8 @@ define(['jquery', 'core/templates', 'core/notification', 'core/key_codes',
             return;
         }
 
+        var pendingPromise = new Pending('core/modal:show');
+
         if (this.hasFooterContent()) {
             this.showFooter();
         } else {
@@ -555,7 +557,8 @@ define(['jquery', 'core/templates', 'core/notification', 'core/key_codes',
             this.attachToDOM();
         }
 
-        this.getBackdrop().done(function(backdrop) {
+        this.getBackdrop()
+        .then(function(backdrop) {
             var currentIndex = this.calculateZIndex();
             var newIndex = currentIndex + 2;
             var newBackdropIndex = newIndex - 1;
@@ -568,7 +571,10 @@ define(['jquery', 'core/templates', 'core/notification', 'core/key_codes',
             this.getModal().focus();
             $('body').addClass('modal-open');
             this.root.trigger(ModalEvents.shown, this);
-        }.bind(this));
+
+            return;
+        }.bind(this))
+        .then(pendingPromise.resolve);
     };
 
     /**
@@ -577,10 +583,6 @@ define(['jquery', 'core/templates', 'core/notification', 'core/key_codes',
      * @method hide
      */
     Modal.prototype.hide = function() {
-        if (!this.isVisible()) {
-            return;
-        }
-
         this.getBackdrop().done(function(backdrop) {
             if (!this.countOtherVisibleModals()) {
                 // Hide the backdrop if we're the last open modal.
