@@ -25,8 +25,8 @@
 
 // NOTE: no MOODLE_INTERNAL test here, this file may be required by behat before including /config.php.
 
-use Behat\Mink\Session as Session,
-    Behat\Mink\Element\NodeElement as NodeElement;
+use Behat\Mink\Element\NodeElement;
+use Behat\Mink\Session;
 
 /**
  * Representation of a form field.
@@ -38,7 +38,10 @@ use Behat\Mink\Session as Session,
  * @copyright  2012 David Monllaó
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
-class behat_form_field {
+class behat_form_field implements behat_session_interface {
+
+    // All of the functionality of behat_base is shared with form fields via the behat_session_trait trait.
+    use behat_session_trait;
 
     /**
      * @var Session Behat session.
@@ -55,6 +58,16 @@ class behat_form_field {
      */
     protected $fieldlocator = false;
 
+    /**
+     * Returns the Mink session.
+     *
+     * @param   string|null $name name of the session OR active session will be used
+     * @return  \Behat\Mink\Session
+     */
+    public function getSession($name = null) {
+        return $this->session;
+    }
+
 
     /**
      * General constructor with the node and the session to interact with.
@@ -107,8 +120,16 @@ class behat_form_field {
         // dealing with a fgroup element.
         $instance = $this->guess_type();
         $instance->field->keyDown($char, $modifier);
-        $instance->field->keyPress($char, $modifier);
-        $instance->field->keyUp($char, $modifier);
+        try {
+            $instance->field->keyPress($char, $modifier);
+            $instance->field->keyUp($char, $modifier);
+        } catch (WebDriver\Exception $e) {
+            // If the JS handler attached to keydown or keypress destroys the element
+            // the later events may trigger errors because form element no longer exist
+            // or is not visible. Ignore such exceptions here.
+        } catch (\Behat\Mink\Exception\ElementNotFoundException $e) {
+            // Other Mink drivers can throw this for the same reason as above.
+        }
     }
 
     /**
@@ -128,6 +149,16 @@ class behat_form_field {
         return $instance->matches($expectedvalue);
     }
 
+    /**
+     * Get the value of an attribute set on this field.
+     *
+     * @param string $name The attribute name
+     * @return string The attribute value
+     */
+    public function get_attribute($name) {
+        return $this->field->getAttribute($name);
+    }
+
     /**
      * Guesses the element type we are dealing with in case is not a text-based element.
      *
@@ -164,6 +195,20 @@ class behat_form_field {
         return get_class($this->session->getDriver()) !== 'Behat\Mink\Driver\GoutteDriver';
     }
 
+    /**
+     * Waits for all the JS activity to be completed.
+     *
+     * @return bool Whether any JS is still pending completion.
+     */
+    protected function wait_for_pending_js() {
+        if (!$this->running_javascript()) {
+            // JS is not available therefore there is nothing to wait for.
+            return false;
+        }
+
+        return behat_base::wait_for_pending_js_in_session($this->session);
+    }
+
     /**
      * Gets the field internal id used by selenium wire protocol.
      *
