@@ -17,15 +17,14 @@
 /**
  * Glossary module version information
  *
- * @package    mod
- * @subpackage glossary
+ * @package mod_glossary
  * @copyright  2011 onwards Eloy Lafuente (stronk7) {@link http://stronk7.com}
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
 
 defined('MOODLE_INTERNAL') || die();
 
-$module->version   = 2012112900;       // The current module version (Date: YYYYMMDDXX)
-$module->requires  = 2012112900;    // Requires this Moodle version
-$module->component = 'mod_glossary';   // Full name of the plugin (used for diagnostics)
-$module->cron      = 0;
+$plugin->version   = 2018051401;       // The current module version (Date: YYYYMMDDXX)
+$plugin->requires  = 2018050800;    // Requires this Moodle version
+$plugin->component = 'mod_glossary';   // Full name of the plugin (used for diagnostics)
+$plugin->cron      = 0;
